@@ -22,6 +22,13 @@ extern "C" {
 #include <stdint.h>
 #include <unistd.h>
 
+/**
+ * File descriptors for a control type.
+ */
+typedef struct powercap_control_type {
+  int enabled;
+} powercap_control_type;
+
 /**
  * File descriptors for a zone.
  */
@@ -47,6 +54,13 @@ typedef struct powercap_constraint {
   int name;
 } powercap_constraint;
 
+/**
+ * Control type file enumeration.
+ */
+typedef enum powercap_control_type_file {
+  POWERCAP_CONTROL_TYPE_FILE_ENABLED
+} powercap_control_type_file;
+
 /**
  * Zone file enumeration.
  */
@@ -72,6 +86,12 @@ typedef enum powercap_constraint_file {
   POWERCAP_CONSTRAINT_FILE_NAME
 } powercap_constraint_file;
 
+/**
+ * Get the filename for a control type file type.
+ * Returns the number of characters written excluding the terminating null byte, or a negative value in case of error.
+ */
+int powercap_control_type_file_get_name(powercap_control_type_file type, char* buf, size_t size);
+
 /**
  * Get the filename for a zone file type.
  * Returns the number of characters written excluding the terminating null byte, or a negative value in case of error.
@@ -84,6 +104,16 @@ int powercap_zone_file_get_name(powercap_zone_file type, char* buf, size_t size)
  */
 int powercap_constraint_file_get_name(powercap_constraint_file type, uint32_t constraint, char* buf, size_t size);
 
+/**
+ * Set the control type's enabled value.
+ */
+int powercap_control_type_set_enabled(const powercap_control_type* control, int val);
+
+/**
+ * Get the control type's enabled value.
+ */
+int powercap_control_type_get_enabled(const powercap_control_type* control, int* val);
+
 /**
  * Get the zone's maximum energy range in microjoules.
  */
