@@ -8,123 +8,82 @@
  */
 #include <assert.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include "powercap.h"
 #include "powercap-common.h"
 #include "powercap-rapl.h"
-#include "powercap-rapl-sysfs.h"
+#include "powercap-sysfs.h"
+
+#ifndef MAX_NAME_SIZE
+  #define MAX_NAME_SIZE 64
+#endif
 
 #define CONTROL_TYPE "intel-rapl"
 
-#define CONSTRAINT_NUM_LONG 0
-#define CONSTRAINT_NUM_SHORT 1
-
 #define CONSTRAINT_NAME_LONG "long_term"
 #define CONSTRAINT_NAME_SHORT "short_term"
 
-#define PP_NAME_CORE "core"
-#define PP_NAME_UNCORE "uncore"
-#define PP_NAME_DRAM "dram"
-#define PP_NAME_PSYS "psys"
+#define ZONE_NAME_PREFIX_PKG "package"
+#define ZONE_NAME_CORE "core"
+#define ZONE_NAME_UNCORE "uncore"
+#define ZONE_NAME_DRAM "dram"
+#define ZONE_NAME_PSYS "psys"
 
-static int rapl_open_zone_file(const uint32_t* zones, uint32_t depth, powercap_zone_file type, int flags, int* fd) {
-  assert(fd != NULL);
-  char buf[PATH_MAX];
-  if ((*fd = open_zone_file(CONTROL_TYPE, zones, depth, type, flags)) < 0) {
-    // for logging purposes only
-    get_zone_file_path(CONTROL_TYPE, zones, depth, type, buf, sizeof(buf));
-    if (errno == ENOENT) {
-      // No such file or directory
-      LOG(DEBUG, "rapl_open_zone_file: access: %s: %s\n", buf, strerror(errno));
-      *fd = 0;
-    } else if (errno == EACCES && type == POWERCAP_ZONE_FILE_ENERGY_UJ) {
-      // special case for energy_uj (it's actually read-only for RAPL)
-      errno = 0;
-      *fd = open_zone_file(CONTROL_TYPE, zones, depth, type, O_RDONLY);
-      if (*fd < 0) {
-        LOG(ERROR, "rapl_open_zone_file: open (RO): %s: %s\n", buf, strerror(errno));
-      }
-    } else {
-      LOG(ERROR, "rapl_open_zone_file: open: %s: %s\n", buf, strerror(errno));
-    }
+static powercap_constraint* get_constraint_by_rapl_name(powercap_rapl_zone_files* fds, const uint32_t* zones, uint32_t depth, uint32_t constraint) {
+  assert(fds != NULL);
+  char name[MAX_NAME_SIZE];
+  if (powercap_sysfs_constraint_get_name(CONTROL_TYPE, zones, depth, constraint, name, sizeof(name)) < 0) {
+    return NULL;
   }
-  return *fd < 0 ? -errno : 0;
-}
-
-static int rapl_open_constraint_file(const uint32_t* zones, uint32_t depth, powercap_constraint_file type, uint32_t constraint, int flags, int* fd) {
-  assert(fd != NULL);
-  char buf[PATH_MAX];
-  if ((*fd = open_constraint_file(CONTROL_TYPE, zones, depth, constraint, type, flags)) < 0) {
-    // for logging purposes only
-    get_constraint_file_path(CONTROL_TYPE, zones, depth, constraint, type, buf, sizeof(buf));
-    if (errno == ENOENT) {
-      // No such file or directory
-      LOG(DEBUG, "rapl_open_constraint_file: access: %s: %s\n", buf, strerror(errno));
-      *fd = 0;
-    } else {
-      LOG(ERROR, "rapl_open_constraint_file: open: %s: %s\n", buf, strerror(errno));
-    }
+  if (!strncmp(name, CONSTRAINT_NAME_LONG, sizeof(CONSTRAINT_NAME_LONG))) {
+    return &fds->constraint_long;
+  } else if (!strncmp(name, CONSTRAINT_NAME_SHORT, sizeof(CONSTRAINT_NAME_SHORT))) {
+    return &fds->constraint_short;
+  } else {
+    LOG(ERROR, "powercap-rapl: Unrecognized constraint name: %s\n", name);
+    errno = EINVAL;
   }
-  return *fd < 0 ? -errno : 0;
+  return NULL;
 }
 
-static int open_zone(const uint32_t* zones, uint32_t depth, powercap_zone* fds, int ro) {
+static int open_all(const uint32_t* zones, uint32_t depth, powercap_rapl_zone_files* fds, int ro) {
   assert(fds != NULL);
-  return rapl_open_zone_file(zones, depth, POWERCAP_ZONE_FILE_MAX_ENERGY_RANGE_UJ, O_RDONLY, &fds->max_energy_range_uj) ||
-         rapl_open_zone_file(zones, depth, POWERCAP_ZONE_FILE_ENERGY_UJ, ro ? O_RDONLY : O_RDWR, &fds->energy_uj) ||
-         rapl_open_zone_file(zones, depth, POWERCAP_ZONE_FILE_MAX_POWER_RANGE_UW, O_RDONLY, &fds->max_power_range_uw) ||
-         rapl_open_zone_file(zones, depth, POWERCAP_ZONE_FILE_POWER_UW, O_RDONLY, &fds->power_uw) ||
-         rapl_open_zone_file(zones, depth, POWERCAP_ZONE_FILE_ENABLED, ro ? O_RDONLY : O_RDWR, &fds->enabled) ||
-         rapl_open_zone_file(zones, depth, POWERCAP_ZONE_FILE_NAME, O_RDONLY, &fds->name);
-}
-
-static int open_constraint(const uint32_t* zones, uint32_t depth, uint32_t constraint, powercap_constraint* fds, int ro) {
-  assert(fds != NULL);
-  return rapl_open_constraint_file(zones, depth, POWERCAP_CONSTRAINT_FILE_POWER_LIMIT_UW, constraint, ro ? O_RDONLY : O_RDWR, &fds->power_limit_uw) ||
-         rapl_open_constraint_file(zones, depth, POWERCAP_CONSTRAINT_FILE_TIME_WINDOW_US, constraint, ro ? O_RDONLY : O_RDWR, &fds->time_window_us) ||
-         rapl_open_constraint_file(zones, depth, POWERCAP_CONSTRAINT_FILE_MAX_POWER_UW, constraint, O_RDONLY, &fds->max_power_uw) ||
-         rapl_open_constraint_file(zones, depth, POWERCAP_CONSTRAINT_FILE_MIN_POWER_UW, constraint, O_RDONLY, &fds->min_power_uw) ||
-         rapl_open_constraint_file(zones, depth, POWERCAP_CONSTRAINT_FILE_MAX_TIME_WINDOW_US, constraint, O_RDONLY, &fds->max_time_window_us) ||
-         rapl_open_constraint_file(zones, depth, POWERCAP_CONSTRAINT_FILE_MIN_TIME_WINDOW_US, constraint, O_RDONLY, &fds->min_time_window_us) ||
-         rapl_open_constraint_file(zones, depth, POWERCAP_CONSTRAINT_FILE_NAME, constraint, O_RDONLY, &fds->name);
-}
-
-static int is_wrong_constraint(const powercap_constraint* fds, const char* expected_name) {
-  assert(fds != NULL);
-  assert(expected_name != NULL);
-  char buf[32];
-  // assume constraint is wrong unless we can prove it's correct
-  return powercap_constraint_get_name(fds, buf, sizeof(buf)) <= 0 ||
-         strncmp(buf, expected_name, sizeof(buf)) != 0;
-}
-
-static int open_all(uint32_t id, uint32_t pp, int is_pp, powercap_rapl_zone_files* fds, int ro) {
-  assert(fds != NULL);
-  powercap_constraint tmp;
-  uint32_t zones[2];
-  uint32_t depth;
-  zones[0] = id;
-  zones[1] = pp;
-  depth = is_pp ? 2 : 1;
-  if (open_zone(zones, depth, &fds->zone, ro) ||
-      open_constraint(zones, depth, CONSTRAINT_NUM_LONG, &fds->constraint_long, ro) ||
-      open_constraint(zones, depth, CONSTRAINT_NUM_SHORT, &fds->constraint_short, ro)) {
-    return 1;
+  char buf[PATH_MAX] = { 0 };
+  powercap_constraint* pc;
+  uint32_t i = 0;
+  if (powercap_zone_open(&fds->zone, buf, sizeof(buf), CONTROL_TYPE, zones, depth, ro)) {
+    LOG(ERROR, "powercap-rapl: %s: %s\n", buf, strerror(errno));
+    return -errno;
   }
-  // verify that constraints aren't reversed
+  // constraint 0 is supposed to be long_term and constraint 1 (if exists) should be short_term
   // note: never actually seen this problem, but not 100% sure it can't happen, so check anyway...
-  if (is_wrong_constraint(&fds->constraint_long, CONSTRAINT_NAME_LONG) &&
-      is_wrong_constraint(&fds->constraint_short, CONSTRAINT_NAME_SHORT)) {
-    LOG(WARN, "open_all: long and short term constraints are out of order for zone ID: %"PRIu32"\n", id);
-    memcpy(&tmp, &fds->constraint_short, sizeof(powercap_constraint));
-    memcpy(&fds->constraint_short, &fds->constraint_long, sizeof(powercap_constraint));
-    memcpy(&fds->constraint_long, &tmp, sizeof(powercap_constraint));
+  while (!powercap_sysfs_constraint_exists(CONTROL_TYPE, zones, depth, i)) {
+    if ((pc = get_constraint_by_rapl_name(fds, zones, depth, i)) == NULL) {
+      return -errno;
+    }
+    // "power_limit_uw" is picked arbitrarily, but it is a required file
+    if (pc->power_limit_uw) {
+      if (depth == 1) {
+        LOG(ERROR, "powercap-rapl: Duplicate constraint detected at zone: %"PRIu32"\n", zones[0]);
+      } else {
+        LOG(ERROR, "powercap-rapl: Duplicate constraint detected at zone: %"PRIu32":%"PRIu32"\n", zones[0], zones[1]);
+      }
+      errno = EINVAL;
+      return -errno;
+    }
+    buf[0] = '\0';
+    if (powercap_constraint_open(pc, buf, sizeof(buf), CONTROL_TYPE, zones, depth, i, ro)) {
+      LOG(ERROR, "powercap-rapl: %s: %s\n", buf, strerror(errno));
+      return -errno;
+    }
+    i++;
   }
+  // powercap_sysfs_constraint_exists returns error code when constraint does not exist - make sure it's not our fault
+  assert(errno != EINVAL);
+  assert(errno != ENOBUFS);
   return 0;
 }
 
@@ -143,7 +102,7 @@ static const powercap_rapl_zone_files* get_files(const powercap_rapl_pkg* pkg, p
       return &pkg->psys;
     default:
       // somebody passed a bad zone type
-      LOG(ERROR, "get_files: Bad powercap_rapl_zone: %d\n", zone);
+      LOG(ERROR, "powercap-rapl: Bad powercap_rapl_zone: %d\n", zone);
       errno = EINVAL;
       return NULL;
   }
@@ -168,7 +127,7 @@ static const powercap_constraint* get_constraint_files(const powercap_rapl_pkg*
       return &fds->constraint_short;
     default:
       // somebody passed a bad constraint type
-      LOG(ERROR, "get_constraint_files: Bad powercap_rapl_constraint: %d\n", constraint);
+      LOG(ERROR, "powercap-rapl: Bad powercap_rapl_constraint: %d\n", constraint);
       errno = EINVAL;
       return NULL;
   }
@@ -194,7 +153,7 @@ static int get_zone_fd(const powercap_rapl_pkg* pkg, powercap_rapl_zone zone, po
     case POWERCAP_ZONE_FILE_NAME:
       return fds->name;
     default:
-      LOG(ERROR, "get_zone_fd: Bad powercap_zone_file: %d\n", file);
+      LOG(ERROR, "powercap-rapl: Bad powercap_zone_file: %d\n", file);
       errno = EINVAL;
       return -errno;
   }
@@ -222,95 +181,95 @@ static int get_constraint_fd(const powercap_rapl_pkg* pkg, powercap_rapl_zone zo
     case POWERCAP_CONSTRAINT_FILE_NAME:
       return fds->name;
     default:
-      LOG(ERROR, "get_constraint_fd: Bad powercap_constraint_file: %d\n", file);
+      LOG(ERROR, "powercap-rapl: Bad powercap_constraint_file: %d\n", file);
       errno = EINVAL;
       return -errno;
   }
 }
 
-static uint32_t get_num_power_planes(uint32_t id) {
-  uint32_t sz = 0;
-  while (!rapl_sysfs_zone_exists(id, sz, 1)) {
-    sz++;
+static powercap_rapl_zone_files* get_files_by_name(powercap_rapl_pkg* pkg, const uint32_t* zones, uint32_t depth) {
+  assert(pkg != NULL);
+  char name[MAX_NAME_SIZE];
+  if (powercap_sysfs_zone_get_name(CONTROL_TYPE, zones, depth, name, sizeof(name)) < 0) {
+    return NULL;
   }
-  return sz;
-}
-
-static ssize_t get_pp_type(uint32_t id, uint32_t pp, powercap_rapl_zone* zone) {
-  assert(zone != NULL);
-  char name[8];
-  ssize_t ret;
-  if ((ret = rapl_sysfs_zone_get_name(id, pp, 1, name, sizeof(name))) < 0) {
-    /* Casting to int causes uninitialized warning in calling function, so we return ssize_t */
-    return ret;
-  }
-  if (!strncmp(name, PP_NAME_CORE, sizeof(PP_NAME_CORE))) {
-    *zone = POWERCAP_RAPL_ZONE_CORE;
-  } else if (!strncmp(name, PP_NAME_UNCORE, sizeof(PP_NAME_UNCORE))) {
-    *zone = POWERCAP_RAPL_ZONE_UNCORE;
-  } else if (!strncmp(name, PP_NAME_DRAM, sizeof(PP_NAME_DRAM))) {
-    *zone = POWERCAP_RAPL_ZONE_DRAM;
-  } else if (!strncmp(name, PP_NAME_PSYS, sizeof(PP_NAME_PSYS))) {
-    *zone = POWERCAP_RAPL_ZONE_PSYS;
+  if (!strncmp(name, ZONE_NAME_PREFIX_PKG, sizeof(ZONE_NAME_PREFIX_PKG) - 1)) {
+    return &pkg->pkg;
+  } else if (!strncmp(name, ZONE_NAME_CORE, sizeof(ZONE_NAME_CORE))) {
+    return &pkg->core;
+  } else if (!strncmp(name, ZONE_NAME_UNCORE, sizeof(ZONE_NAME_UNCORE))) {
+    return &pkg->uncore;
+  } else if (!strncmp(name, ZONE_NAME_DRAM, sizeof(ZONE_NAME_DRAM))) {
+    return &pkg->dram;
+  } else if (!strncmp(name, ZONE_NAME_PSYS, sizeof(ZONE_NAME_PSYS))) {
+    return &pkg->psys;
   } else {
+    LOG(ERROR, "powercap-rapl: Unrecognized zone name: %s\n", name);
     errno = EINVAL;
-    return -errno;
   }
-  return 0;
+  return NULL;
 }
 
-uint32_t powercap_rapl_get_num_packages(void) {
+int powercap_rapl_control_is_supported(void) {
+  int ret = powercap_sysfs_control_type_exists(CONTROL_TYPE);
+  return ret ? (errno == ENOSYS ? 0 : ret) : 1;
+}
+
+int powercap_rapl_control_is_enabled(void) {
+  uint32_t enabled;
+  int ret = powercap_sysfs_control_type_get_enabled(CONTROL_TYPE, &enabled);
+  return ret ? ret : (enabled ? 1 : 0);
+}
+
+int powercap_rapl_control_set_enabled(int val) {
+  return powercap_sysfs_control_type_set_enabled(CONTROL_TYPE, (uint32_t) val);
+}
+
+uint32_t powercap_rapl_get_num_instances(void) {
   uint32_t n = 0;
-  while (!rapl_sysfs_zone_exists(n, 0, 0)) {
+  while (!powercap_sysfs_zone_exists(CONTROL_TYPE, &n, 1)) {
     n++;
   }
   if (!n) {
-    LOG(ERROR, "powercap_rapl_get_num_packages: No top-level zones found - is the intel_rapl kernel module loaded?\n");
+    LOG(ERROR, "powercap-rapl: No top-level "CONTROL_TYPE" zones found - is its kernel module loaded?\n");
     errno = ENOENT;
   }
   return n;
 }
 
+uint32_t powercap_rapl_get_num_packages(void) {
+  return powercap_rapl_get_num_instances();
+}
+
 int powercap_rapl_init(uint32_t id, powercap_rapl_pkg* pkg, int read_only) {
   int ret;
-  ssize_t sret;
   int err_save;
-  uint32_t i;
-  uint32_t npp;
-  powercap_rapl_zone type;
+  uint32_t zones[2] = { id, 0 };
+  powercap_rapl_zone_files* files;
   if (pkg == NULL) {
     errno = EINVAL;
     return -errno;
   }
+  // first need the parent zone
+  if ((files = get_files_by_name(pkg, zones, 1)) == NULL) {
+    return -errno;
+  }
   // force all fds to 0 so we don't try to operate on invalid descriptors
   memset(pkg, 0, sizeof(powercap_rapl_pkg));
-  // first populate zone and package power zone
-  if (!(ret = open_all(id, 0, 0, &pkg->pkg, read_only))) {
-    // get a count of subordinate power zones in this package
-    npp = get_num_power_planes(id);
-    // now get all power zones
-    for (i = 0; i < npp && !ret; i++) {
-      if ((sret = get_pp_type(id, i, &type))) {
-        ret = (int) sret;
-        break;
-      }
-      switch (type) {
-        case POWERCAP_RAPL_ZONE_CORE:
-          ret = open_all(id, i, 1, &pkg->core, read_only);
-          break;
-        case POWERCAP_RAPL_ZONE_UNCORE:
-          ret = open_all(id, i, 1, &pkg->uncore, read_only);
-          break;
-        case POWERCAP_RAPL_ZONE_DRAM:
-          ret = open_all(id, i, 1, &pkg->dram, read_only);
-          break;
-        case POWERCAP_RAPL_ZONE_PSYS:
-          ret = open_all(id, i, 1, &pkg->psys, read_only);
-          break;
-        case POWERCAP_RAPL_ZONE_PACKAGE:
-        default:
-          assert(0);
-          break;
+  // first populate parent zone
+  if (!(ret = open_all(zones, 1, files, read_only))) {
+    // get subordinate power zones
+    while(!powercap_sysfs_zone_exists(CONTROL_TYPE, zones, 2) && !ret) {
+      if ((files = get_files_by_name(pkg, zones, 2)) == NULL) {
+        ret = -errno;
+      } else if (files->zone.name) {
+        // zone has already been opened ("name" is picked arbitrarily, but it is a required file)
+        LOG(ERROR, "powercap-rapl: Duplicate zone type detected at %"PRIu32":%"PRIu32"\n", zones[0], zones[1]);
+        errno = EBUSY;
+        ret = -errno;
+      } else {
+        ret = open_all(zones, 2, files, read_only);
+        zones[1]++;
       }
     }
   }
@@ -322,41 +281,12 @@ int powercap_rapl_init(uint32_t id, powercap_rapl_pkg* pkg, int read_only) {
   return ret;
 }
 
-static int powercap_rapl_close(int fd) {
-  return (fd > 0 && close(fd)) ? -errno : 0;
-}
-
-static int fds_destroy_zone(powercap_zone* fds) {
-  assert(fds != NULL);
-  int ret = 0;
-  ret |= powercap_rapl_close(fds->max_energy_range_uj);
-  ret |= powercap_rapl_close(fds->energy_uj);
-  ret |= powercap_rapl_close(fds->max_power_range_uw);
-  ret |= powercap_rapl_close(fds->power_uw);
-  ret |= powercap_rapl_close(fds->enabled);
-  ret |= powercap_rapl_close(fds->name);
-  return ret;
-}
-
-static int fds_destroy_zone_constraint(powercap_constraint* fds) {
-  assert(fds != NULL);
-  int ret = 0;
-  ret |= powercap_rapl_close(fds->power_limit_uw);
-  ret |= powercap_rapl_close(fds->time_window_us);
-  ret |= powercap_rapl_close(fds->max_power_uw);
-  ret |= powercap_rapl_close(fds->min_power_uw);
-  ret |= powercap_rapl_close(fds->max_time_window_us);
-  ret |= powercap_rapl_close(fds->min_time_window_us);
-  ret |= powercap_rapl_close(fds->name);
-  return ret;
-}
-
 static int fds_destroy_all(powercap_rapl_zone_files* files) {
   assert(files != NULL);
   int ret = 0;
-  ret |= fds_destroy_zone(&files->zone);
-  ret |= fds_destroy_zone_constraint(&files->constraint_long);
-  ret |= fds_destroy_zone_constraint(&files->constraint_short);
+  ret |= powercap_zone_close(&files->zone);
+  ret |= powercap_constraint_close(&files->constraint_long);
+  ret |= powercap_constraint_close(&files->constraint_short);
   return ret;
 }
 
@@ -373,12 +303,12 @@ int powercap_rapl_destroy(powercap_rapl_pkg* pkg) {
 }
 
 int powercap_rapl_is_zone_supported(const powercap_rapl_pkg* pkg, powercap_rapl_zone zone) {
-  // long constraint is always required for zones
-  return powercap_rapl_is_constraint_supported(pkg, zone, POWERCAP_RAPL_CONSTRAINT_LONG);
+  // POWERCAP_ZONE_FILE_NAME is picked arbitrarily, but it is a required file
+  return powercap_rapl_is_zone_file_supported(pkg, zone, POWERCAP_ZONE_FILE_NAME);
 }
 
 int powercap_rapl_is_constraint_supported(const powercap_rapl_pkg* pkg, powercap_rapl_zone zone, powercap_rapl_constraint constraint) {
-  // power limit is always required for constraints
+  // POWERCAP_CONSTRAINT_FILE_POWER_LIMIT_UW is picked arbitrarily, but it is a required file
   return powercap_rapl_is_constraint_file_supported(pkg, zone, constraint, POWERCAP_CONSTRAINT_FILE_POWER_LIMIT_UW);
 }
 
